from openbharatocr.ocr.pan import pan
from openbharatocr.ocr.aadhaar import front_aadhaar, back_aadhaar
from openbharatocr.ocr.driving_licence import driving_licence
from openbharatocr.ocr.passport import passport
from openbharatocr.ocr.voter_id import voter_id_front, voter_id_back
from openbharatocr.ocr.vehicle_registration import vehicle_registration
from openbharatocr.ocr.water_bill import water_bill
